"""Lock a database to a certain timestamp.

Calling the methods on a model has the effect that queries to the model will
only consider data created up to the specified timestamp (older or equals).
Locks are temporary, the lock is effective until the model is unlocked or
closed.

The timelock turns the model temporarily read-only. While a model is
timelocked, ALL WRITES WILL FAIL with an sqlite3.OperationalError


Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016
"""
# exports
__all__ = ('timelock_lock', 'timelock_change', 'timelock_unlock')


## code ##

def timelock_lock(model, timestamp):
    """Restrict database to operate on items older as *timestamp*."""
    # TODO: Watch out for commit behaviour!
    timestamp = float(timestamp)
    model.conn.execute('CREATE TEMPORARY VIEW image   AS SELECT * FROM main.image   WHERE t_image_create <= {}'.format(timestamp))
    model.conn.execute('CREATE TEMPORARY VIEW tag     AS SELECT * FROM main.tag     WHERE t_created      <= {}'.format(timestamp))
    model.conn.execute('CREATE TEMPORARY VIEW img_tag AS SELECT * FROM main.img_tag WHERE t_created      <= {}'.format(timestamp))
    return model

def timelock_change(model, timestamp):
    """Change the locked *timestamp*."""
    timelock_unlock(model)
    timelock_init(model, timestamp)
    return model

def timelock_unlock(model):
    """Release time restrictions from *model*."""
    model.conn.execute('DROP VIEW image')
    model.conn.execute('DROP VIEW tag')
    model.conn.execute('DROP VIEW img_tag')
    return model

## EOF ##
