"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# STANDARD IMPORTS
from hashlib import sha1
from time import time
from os.path import expanduser

# INNER-MODULE IMPORTS
from updater import Updater
from tagit.basics import fst

# EXPORTS
__all__ = ('Update_5', )

## CODE ##

class Updater_5(Updater):
    def apply(self, conn):
        # apply update
        conn.executescript('''
-- create new tables
CREATE TABLE metadata (
    id TEXT,
    name TEXT,
    t_created INTEGER unsigned DEFAULT 0
);
        ''') # Commits before script execution

        # metadata
        id_, name, path = conn.execute('PRAGMA database_list').fetchone() # id_ = 0, name = 'main', path = file path or '' (in-memory)
        path = path or 'GENERIC'
        instid = sha1(path + str(time())).hexdigest()
        conn.execute('INSERT INTO metadata (id, name, t_created) VALUES (?, ?, ?)', (instid, path, time()))
        sender = (instid, path)

        # Move data to DataLog
        from tagit.datalog import DataLog
        d = DataLog(expanduser('~/.taglog.db')) # FIXME: Respect settings!
        for sid, timestamp in conn.execute('SELECT id, t_logged FROM search_history'):
            inactive = map(fst, conn.execute('SELECT token FROM search_token WHERE active = 0 AND search = ?', (sid, )).fetchall())
            active   = map(fst, conn.execute('SELECT token FROM search_token WHERE active = 1 AND search = ?', (sid, )).fetchall())

            d.log_search(sender, active, inactive, timestamp)

        # finish update
        conn.executescript('''
-- remove old tables
DROP TABLE search_history;
DROP TABLE search_token;

-- Update version
PRAGMA user_version = 5;

-- EOF --
        ''') # Commits before script execution


## EOF ##
