"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
import json

# INNER-MODULE IMPORTS

# EXPORTS
__all__ = ('token_factory', 'token_from_string', 'Token', 'Token_SingleValue', 'Token_SingleValue_Parsed')

## CODE ##

class Token(object):
    def display(self):
        """Display name (abbreviation)."""
        raise NotImplementedError()
    def edit(self):
        """Return editable text."""
        raise NotImplementedError()
    def update(self, string):
        """Update from a string. Returns self."""
        raise NotImplementedError()
    def get(self):
        """Return raw value."""
        raise NotImplementedError()
    def __eq__(self, other):
        return type(self) == type(other) and self.get() == other.get()
    @staticmethod
    def from_string(string):
        """Build from string."""
        raise NotImplementedError()
    @staticmethod
    def match(obj):
        """Compare type."""
        # Usually: return isinstance(obj, Token_XXYY)
        raise NotImplementedError()

class Token_SingleValue(Token):
    def __init__(self, init=None):
        self._value = init
    def get(self):
        return self._value
    def edit(self):
        return self._value
    def update(self, string):
        self._value = string
        return self

class Token_SingleValue_Parsed(Token_SingleValue):
    def edit(self):
        return json.dumps(self._value)
    def update(self, string):
        self._value = json.loads(string)
        return self

_token_builder = {}

def register_token(name, factory):
    #print "Register token", name
    _token_builder[name] = factory

def token_factory(type_):
    try:
        cls = _token_builder[type_]
        return cls
    except IndexError:
        raise Exception('token type unknown')

def token_from_string(type_, *args, **kwargs):
    try:
        cls = _token_builder[type_]
        return cls.from_string(*args, **kwargs)
    except IndexError:
        raise Exception('token type unknown')

## EOF ##
