"""Dialogue to create a directory.
"""
# imports
from kivy.properties import StringProperty
from dialogue import Dialogue
from kivy.lang import Builder
from os.path import join, dirname, exists, isdir
from os import listdir

from dlg_text import LabelDialogue
from dlg_error import ErrorDialogue

# exports
__all__ = ('DirCreatorDialogue', )

# Load kv
Builder.load_file(join(dirname(__file__), 'dlg_new_dir.kv'))

# Classes
class DirCreatorDialogue(Dialogue):
    """Dialogue with a file browser to select a file.

    Pass the default text as **text**.

    >>> DirCreatorDialogue(text='Hello world').open()

    """

    text = StringProperty('')

    def ok(self):
        if exists(self.path):
            if not isdir(self.path):
                dlg = ErrorDialogue(text='Please select a directory')
                dlg.open()
            elif len(listdir(self.path)) > 0:
                dlg = LabelDialogue(text='The directory is not empty. Files might be overwritten. Continue?')
                dlg.bind(on_ok=lambda obj: super(DirCreatorDialogue, self).ok())
                dlg.open()
            else: # Empty directory
                super(DirCreatorDialogue, self).ok()
        else:
            super(DirCreatorDialogue, self).ok()

## EOF ##
