"""Sidebar.

Where the information flows from.

The *VSidebar* holds the sideboxes together, adds and removes them.

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
from os.path import join, dirname
from kivy.lang import Builder
from kivy.uix.gridlayout import GridLayout
from kivy.logger import Logger
from kivy.uix.splitter import Splitter
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.image import Image
from kivy.properties import StringProperty
#from kivy.uix.scrollview import ScrollView # FIXME: Provokes segfault

# INNER-MODULE IMPORTS
from ..controller.controller import Controller
from sidebox import VSidebox, sidebox_factory
from tagit.external import Tooltip

# EXPORTS
__all__ = ('VSidebar', )

# LOAD KV
Builder.load_file(join(dirname(__file__), 'sidebar.kv'))

# CLASSES
class VSidebar(GridLayout):
    """Load configured sideboxes, present them stacked.
    """
    def __init__(self, *args, **kwargs):
        super(VSidebar, self).__init__(*args, **kwargs)
        self._last_wx = None

    def on_parent(self, *args):
        """Set the controller factory.
        Reinitializes parts of the browser.
        """
        self.controller = self.get_controller(Controller)
        if self.controller is not None:
            boxes = self.controller.settings.trace('view', 'sidebox', [])
            for box in boxes:
                caption, content = sidebox_factory(box)
                wx = Collapsable(text=caption, height=content.height+15)
                wx.add_widget(content)
                self.add_widget(wx)

        super(VSidebar, self).on_parent(*args)

        # Propagate initialization
        if self._last_wx is not None:
            self.on_widget_change(*self._last_wx)

    def on_widget_change(self, instance, widget):
        """Re-initialize the Sideboxes to a new mainWidget.
        """
        self._last_wx = (instance, widget)
        Logger.debug("Reinitialize Sideboxes")
        for wx in self.children:
            wx.on_widget_change(instance, widget)

class VSidebox_Button(ButtonBehavior, Image, Tooltip):
    tooltip = StringProperty('')
    def on_tooltip(self, callee, text):
        self.set_tooltip(text)

#class VBoxScroll(ScrollView, VSidebox):
#    def on_widget_change(self, instance, widget):
#        self.children[-1].on_widget_change(instance, widget)


class Collapsable(GridLayout):
    def __init__(self, *args, **kwargs):
        super(Collapsable, self).__init__(*args, **kwargs)
        self._col_child = None
    def on_widget_change(self, inst, wx):
        if len(self.children) > 1:
            self.children[0].on_widget_change(inst, wx)
    def collapse(self):
        if len(self.children) > 1:
            self._col_child, self._col_height = self.children[0], self.height
            self.remove_widget(self._col_child)
            self.height = 15

        self._collapsed = True
        return True

    def expand(self):
        if self._col_child is not None:
            self.height = self._col_height
            self.add_widget(self._col_child)
        self._collapsed = False

        return True

## EOF ##
