"""Sideboxes.

A *Sidebox* is initialized through the *on_widget_change* method. There,
it has to get the controller factories together, initialize and attach
itself to events from the mainWidget. Whenever such an event is triggered,
a content update is to be made.

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
import warnings

# INNER-MODULE IMPORTS
from sidebox import VSidebox
from db_management import VSidebox_DB_Management
from tags_browser import VSidebox_Tags_Browser
from tags_selection import VSidebox_Tags_Selection
from tags_cursor import VSidebox_Tags_Cursor
from tags_suggested import VSidebox_Tags_Suggested
from info import VSidebox_Info_Cursor
from pgm import VSidebox_Program_Control
from export import VSidebox_Image_Export

# EXPORTS
__all__ = ('VSidebox', 'VSidebox_DB_Management', 'sidebox_factory')

## CODE ##

_sidebox_factory_lut = {
      'DB_Management'  : ('Database', VSidebox_DB_Management)
    , 'Tags_Browser'   : ('Tags', VSidebox_Tags_Browser)
    , 'Tags_Selection' : ('Tags', VSidebox_Tags_Selection)
    , 'Tags_Cursor'    : ('Tags', VSidebox_Tags_Cursor)
    , 'Tags_Suggested' : ('Suggestions', VSidebox_Tags_Suggested)
    , 'Program_Control': ('Program', VSidebox_Program_Control)
    , 'Info_Cursor'    : ('Image infos', VSidebox_Info_Cursor)
    , 'Image_Export'   : ('Export', VSidebox_Image_Export)
    }

def register_sidebox(name, caption, factory):
    """Register a sidebox factory."""
    if name in _sidebox_factory_lut:
        warnings.warn('Overwriting Sidebox factory ({})'.format(name))
    _sidebox_factory_lut[name] = (caption, factory)

def sidebox_factory(name, *args, **kwargs):
    """Get a sidebox factory for *name*.
    Additional arguments are passed to the constructor.
    """
    if name not in _sidebox_factory_lut:
        raise Exception('Sidebox {} unknown'.format(name))

    caption, factory = _sidebox_factory_lut[name]
    return caption, factory(*args, **kwargs)


## EOF ##
