"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
from os.path import join, dirname
from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.logger import Logger

# INNER-MODULE IMPORTS
from sidebox import VSidebox
from ...controller.sidebox import CSidebox_Image_Export
from ..dialogues import DirCreatorDialogue, ErrorDialogue

# EXPORTS
__all__ = ('VSidebox_Image_Export', )

# LOAD KV
Builder.load_file(join(dirname(__file__), 'export.kv'))

# CLASSES

class VSidebox_Image_Export(BoxLayout, VSidebox):
    """Export images to a directory."""
    def on_widget_change(self, instance, widget):
        self.controller = widget.get_controller(CSidebox_Image_Export, wx=self, browser=widget.browser)

    def error(self, text):
        dlg = ErrorDialogue(text=text)
        dlg.open()

    def on_export(self, *args):
        dlg = DirCreatorDialogue(text='Please select a target folder')
        dlg.bind(on_ok=lambda obj: self.request_export(obj.path))
        dlg.open()

    def request_export(self, path):
        Logger.debug('Export to {}'.format(path))

        method = [c.text for c in self.method.children if c.state == 'down']
        method = len(method) > 0 and method[0] or 'copy'
        if method in ('sym', 'hard'): method += 'link'

        target = path
        structure = not self.structure.state == 'down'

        self.controller.export(
                method,
                target,
                structure
        )

## EOF ##
