"""

Part of the tagit module.
A copy of the license is provided with the project.
Author: Matthias Baumgartner, 2016

"""
# IMPORTS
from os.path import join, dirname
from kivy.lang import Builder
from kivy.uix.boxlayout import BoxLayout
from kivy.logger import Logger
from kivy.resources import resource_find

# INNER-MODULE IMPORTS
from sidebox import VSidebox
from ...controller.sidebox import CSidebox_Program_Control
from ..dialogues import FilePickerDialogue, FileCreatorDialogue

# EXPORTS
__all__ = ('VSidebox_Program_Control', )

# LOAD KV
Builder.load_file(join(dirname(__file__), 'pgm.kv'))

# CLASSES
class VSidebox_Program_Control(BoxLayout, VSidebox):
    """A bunch of buttons to control the database."""

    PAUSE = ('"', resource_find('30/pause.png'), 'Pause the program')
    START = ('>', resource_find('30/start.png'), 'Run the program')
    PICK  = ('L', resource_find('30/open.png'), 'Select a program')
    LOAD  = ('O', resource_find('30/open.png'), 'Load a program state')
    SAVE  = ('S', resource_find('30/save.png'), 'Save the current state')

    def __init__(self, *args, **kwargs):
        kwargs['height'] = 60
        super(VSidebox_Program_Control, self).__init__(*args, **kwargs)

    def on_widget_change(self, instance, widget):
        self.controller = widget.get_controller(CSidebox_Program_Control, wx=self, main_widget=widget)
        self._run_clbk = self.start
        self._load_clbk = self.load

    def select(self, *args):
        """Select a program.
        """
        Logger.debug("Loading the BFS programme")
        self.controller.select('BFS')
        self.is_initialized = True

    def load(self):
        """Load a program state.
        """
        dlg = FilePickerDialogue(text='Select a program state to load.')
        dlg.filters = ["*.snap"]
        dlg.bind(on_ok=lambda obj: self.controller.load(obj.path))
        dlg.open()

    def save(self):
        Logger.debug("Saving the program")
        dlg = FileCreatorDialogue(text='Select a file to save the program to.')
        dlg.filters = ["*.snap"]
        def addsnap(pth):
            return pth.endswith('.snap') and pth or pth + '.snap'

        dlg.bind(on_ok=lambda obj: self.controller.save(addsnap(obj.path)))
        dlg.open()

    def on_load_btn(self, *args):
        self._load_clbk()

    def on_run_btn(self, *args):
        self._run_clbk()

    def show_play(self, next_press):
        btn = self.run_btn
        btn.text, btn.source, btn.tooltip = self.START
        self._run_clbk = next_press

    def show_pause(self):
        btn = self.run_btn
        btn.text, btn.source, btn.tooltip = self.PAUSE
        self._run_clbk = self.pause

    def show_load(self):
        btn = self.load_btn
        btn.text, btn.source, btn.tooltip = self.LOAD
        self._load_clbk = self.load

    def show_save(self):
        btn = self.load_btn
        btn.text, btn.source, btn.tooltip = self.SAVE
        self._load_clbk = self.save

    def start(self):
        Logger.debug("Starting the program")
        self.controller.start()
        return True

    def stop(self):
        Logger.debug("Stopping the program")
        self.controller.stop()
        return True

    def pause(self):
        Logger.debug("Pausing the program")
        self.controller.pause()
        return True

    def resume(self):
        Logger.debug("Resuming the program")
        self.controller.resume()
        return True


## EOF ##
